<?php

namespace Drupal\decorative_images\Hook;

use Drupal\Core\Field\FieldConfigInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Handle field widget hooks.
 */
class DecorativeWidgetFormAlter {

  /**
   * Alters the 'Decorative' checkbox, if enabled in the field settings.
   */
  public static function alterForm(array &$element, FormStateInterface $form_state, array $context): void {

    // Check if decorative is enabled.
    $field_definition = $context['items']?->getFieldDefinition();
    if (!$field_definition instanceof FieldConfigInterface) {
      return;
    }

    if (!$field_definition->getThirdPartySetting('decorative_images', 'decorative_enabled', FALSE)) {
      return;
    }

    // Add validation if required.
    if ($field_definition->getThirdPartySetting('decorative_images', 'decorative_or_alternative_required', FALSE)) {
      $element['#element_validate'][] = [self::class, 'validateForm'];
    }

    // Check for a file ID.
    $field_values = self::getSubmission($element, $form_state, $context);
    $is_decorative = $field_values['is_decorative'] ?? NULL;
    $target_id = $field_values['fids'][0] ?? NULL;

    // Load the decorative value.
    $decorative_value = $is_decorative ?? \Drupal::service('keyvalue')->get('decorative_images')->get($target_id) ?? 0;

    // Append form element.
    $element['is_decorative'] = [
      '#type' => 'checkbox',
      '#title' => t('Decorative image'),
      '#default_value' => $decorative_value,
    ];

  }

  /**
   * Get the submitted values for the field.
   */
  public static function getSubmission(array $element, FormStateInterface $form_state, array $context): ?array {

    $delta = $context['delta'];
    $field_name = $context['items']->getName();
    $values = $form_state->getValues();

    // Check submitted form value.
    if (!empty($values[$field_name][$delta]['fids'][0])) {
      return $values[$field_name][$delta];
    }

    // Check submitted media value.
    if (!empty($values['media'][0]['fields'][$field_name][$delta]['fids'][0])) {
      return $values['media'][0]['fields'][$field_name][$delta];
    }

    // Fallback to initial value.
    if (!empty($element['#default_value']['fids'][0])) {
      return $element['#default_value'];
    }

    return NULL;
  }

  /**
   * Custom validation function for the image widget.
   */
  public static function validateForm($element, FormStateInterface $form_state): void {

    $trigger = $form_state->getTriggeringElement();
    $name = $trigger['#name'] ?? '';

    // Confirm we have a submitted form.
    if (!$form_state->isSubmitted()) {
      return;
    }

    // Confirm this isn't the upload/ remove action.
    if (stripos($name, 'upload') || stripos($name, 'remove')) {
      return;
    }

    $values = $form_state->getValue($element['#parents'] ?? []);

    if (!empty($values['fids']) && empty($values['alt']) && !$values['is_decorative']) {
      $form_state->setError($element['alt'], t('Alternative text or the decorative option is required.'));
    }
  }

}
